import * as fc from 'fast-check'
import getProminentColors, { GetProminentColorsConfig } from './getProminentColors'

jest.mock('get-pixels', () => () => [[255, 255, 255]])
jest.mock('get-rgba-palette', () => ({
  bins: () => [
    {
      color: [255, 255, 255],
      size: 42069,
      amount: 0.42069,
    }
  ]
}))

it('should exist', () => {
  expect(getProminentColors).toBeDefined()
})

it('should be a callable', () => {
  expect(typeof getProminentColors).toBe('function')
})

it('should accept 1 argument', () => {
  expect(getProminentColors).toHaveLength(1)
})

it('should throw an error for invalid config', () => {
  fc.assert(
    fc.property(
      fc.anything().filter(s => !(typeof s === 'object' && s !== null && 'count' in s)),
      notAnObject => {
        expect(() => getProminentColors(notAnObject as GetProminentColorsConfig)).toThrow(TypeError)
      }
    )
  )
})

it('should throw an error for non-number arguments as count', () => {
  fc.assert(
    fc.property(
      fc.anything().filter(s => typeof s !== 'number'),
      notANumber => {
        expect(() => getProminentColors({ count: notANumber as number })).toThrow(TypeError)
      }
    )
  )
})

it('should throw an error for NaN as count', () => {
  expect(() => getProminentColors({ count: NaN })).toThrow(RangeError)
})

it('should throw an error for 0 as count', () => {
  expect(() => getProminentColors({ count: 0 })).toThrow(RangeError)
})

it('should throw an error for negative values as count', () => {
  fc.assert(
    fc.property(
      fc.nat().filter(n => n !== 0),
      anAbsoluteNumber => {
        expect(() => getProminentColors({ count: -anAbsoluteNumber })).toThrow(RangeError)
      }
    )
  )
})

describe('on valid `count` values', () => {
  it('should throw an error for non-string sources', () => {
    fc.assert(
      fc.property(
        fc.anything().filter(v => typeof v !== 'string'),
        notAString => {
          expect(() => getProminentColors({ count: 1 })(notAString as string)).toThrow(TypeError)
        }
      )
    )
  })

  it('should throw an error for blank strings as sources', () => {
    fc.assert(
      fc.property(
        fc.nat(20),
        blankStringLength => {
          let blankString = ''

          for (let i = 0; i < blankStringLength; i += 1) {
            blankString += ' '
          }

          expect(() => getProminentColors({ count: 1 })(blankString)).toThrow(RangeError)
        }
      )
    )
  })

  it('should return an array of objects with color attributes for valid sources', () => {
    fc.assert(
      fc.property(
        fc.string(1, 20).filter(s => s.trim().length > 0),
        aString => {
          const result = getProminentColors({ count: 1, })(aString)
          expect(result.every(s => 'color' in s)).toBe(true)
        }
      )
    )
  })

  it('should return an array of objects with size attributes for valid sources', () => {
    fc.assert(
      fc.property(
        fc.string(1, 20).filter(s => s.trim().length > 0),
        aString => {
          const result = getProminentColors({ count: 1, })(aString)
          expect(result.every(s => 'size' in s)).toBe(true)
        }
      )
    )
  })

  it('should return an array of objects with amount attributes for valid sources', () => {
    fc.assert(
      fc.property(
        fc.string(1, 20).filter(s => s.trim().length > 0),
        aString => {
          const result = getProminentColors({ count: 1, })(aString)
          expect(result.every(s => 'amount' in s)).toBe(true)
        }
      )
    )
  })
})
