import getPixels from 'get-pixels'
import getRgbaPalette from 'get-rgba-palette'

type PaletteColor = [number, number, number]

type Color = {
  red: number
  green: number
  blue: number
}

interface PaletteBin<ColorFormat> {
  color: ColorFormat
  size: number,
  amount: number
}

export type GetProminentColorsConfig = {
  count: number
}

type GetProminentColors = (params: GetProminentColorsConfig) => (src: string) => PaletteBin<Color>[]

const getProminentColors: GetProminentColors = params => {
  if (typeof params as unknown !== 'object') {
    throw TypeError('Config must be an object.')
  }

  if (
    params === null
    || typeof params.count as unknown !== 'number'
  ) {
    throw TypeError('Count must be a number.')
  }

  if (isNaN(params.count)) {
    throw RangeError('Config cannot be NaN.')
  }

  if (params.count <= 0) {
    throw RangeError('Only positive values allowed for count.')
  }

  return src => {
    if (typeof src as unknown !== 'string') {
      throw TypeError('Source must be a string.')
    }

    if (src.trim().length < 1) {
      throw RangeError('Source must be non-empty.')
    }

    const pixels: PaletteColor[] = getPixels(src)
    const colors: PaletteBin<PaletteColor>[] = getRgbaPalette.bins(pixels, params.count)
    return colors.map(bin => {
      const { color, ...etcBin } = bin
      const [red, green, blue] = color

      return {
        color: {
          red,
          green,
          blue
        },
        ...etcBin
      }
    })
  }
}

export default getProminentColors
